library(tidyverse)
library(parallel)
library(rms)
library(rlist)

# data generation
gen_data <- function(n, p=0.5, beta=c(1,-0.5), sigma=1){
  x1 <- rbinom(n, 1, prob=p)
  x2 <- rnorm(n, 0, 1)
  log.y <- rnorm(n, beta[1] * x1 + beta[2] * x2, sigma)
  data <- data.frame(y=log.y, x1=x1, x2=x2)
  return(data)
}

# mean function for CPM
mean.orm <- function(mod, new.data, se=TRUE){
  if(is.null(mod$yunique)) {
    stop("Need to set x=TRUE and y=TRUE for orm") 
  } else{
    order.y <- exp(mod$yunique) # modify this specific for this case
    n.alpha <- length(order.y)-1
    xb <- as.matrix(new.data) %*% matrix(coef(mod)[colnames(new.data)])
    m.alpha <- mod$coef[1:n.alpha]
    lb <- t(outer(m.alpha, xb, "+")[,,1])
    m.s <- mod$trans$cumprob(lb)
    m.f <- t(apply(m.s, 1, FUN=function(x) c(1,x[1:n.alpha]) - c(x[1:n.alpha], 0)))
    m.mean <- apply(m.f, 1, FUN=function(x) sum(x*order.y))
    
    if(se){
      if(mod$family=="logistic") mod$trans$deriv <- function(x) exp(-x)/(1+exp(-x))^2
      if(mod$family=='probit') mod$trans$deriv <- function(x) dnorm(x)
      if(mod$family=='loglog') mod$trans$deriv <- function(x) exp(-x - exp(-x))
      if(mod$family=='cloglog') mod$trans$deriv <- function(x) exp(x - exp(x))
      
      dmean.dalpha <- t(apply(mod$trans$deriv(lb), 
                              1, FUN=function(x) x*(order.y[2:length(order.y)] - order.y[1:n.alpha])))
      dmean.dbeta <- apply(dmean.dalpha, 1, sum)*as.matrix(new.data)
      dmean.dtheta <- cbind(dmean.dalpha, dmean.dbeta)   
      mean.var <-diag(dmean.dtheta%*%solve(mod$info.matrix)%*%t(dmean.dtheta))
      mean.se <- sqrt(mean.var)   
      result <- cbind(m.mean, mean.se)
      ci <- t(apply(result, 1, FUN=function(x) c(x[1]- qnorm(0.975)*x[2],
                                                 x[1]+ qnorm(0.975)*x[2])))
      result <- cbind(result, ci)
      colnames(result) <- c("est", "se", "lb", "ub")
    } else{
      result <- matrix(m.mean)
      colnames(result) <- c("est")
    }
    
    
    return(result)
  } 
}

# mean function for Zeng
mean.zeng <- function(mod, new.data, se=TRUE, L, U){
  if(is.null(mod$yunique)) {
    stop("Need to set x=TRUE and y=TRUE for orm") 
  } else{
    ind <- which((mod$yunique > L & mod$yunique < U) == 1) # index for [L,U]
    ind_beta <- which(names(coef(mod)) %in% colnames(new.data))
    order.y <- exp(mod$yunique[ind]) # modify this specific for this case
    n.alpha <- length(order.y)-1
    xb <- as.matrix(new.data) %*% matrix(coef(mod)[colnames(new.data)])
    m.alpha <- mod$coef[1:n.alpha]
    lb <- t(outer(m.alpha, xb, "+")[,,1])
    m.s <- mod$trans$cumprob(lb)
    m.f <- t(apply(m.s, 1, FUN=function(x) c(1,x[1:n.alpha]) - c(x[1:n.alpha], 0)))
    m.mean <- apply(m.f, 1, FUN=function(x) sum(x*order.y))
    
    if(se){
      if(mod$family=="logistic") mod$trans$deriv <- function(x) exp(-x)/(1+exp(-x))^2
      if(mod$family=='probit') mod$trans$deriv <- function(x) dnorm(x)
      if(mod$family=='loglog') mod$trans$deriv <- function(x) exp(-x - exp(-x))
      if(mod$family=='cloglog') mod$trans$deriv <- function(x) exp(x - exp(x))
      
      dmean.dalpha <- t(apply(mod$trans$deriv(lb), 
                              1, FUN=function(x) x*(order.y[2:length(order.y)] - order.y[1:n.alpha])))
      dmean.dbeta <- apply(dmean.dalpha, 1, sum)*as.matrix(new.data)
      dmean.dtheta <- cbind(dmean.dalpha, dmean.dbeta)   
      mean.var <-diag(dmean.dtheta %*% 
                        solve(mod$info.matrix)[c(ind[-length(ind)],ind_beta), c(ind[-length(ind)],ind_beta)] 
                      %*% t(dmean.dtheta))
      mean.se <- sqrt(mean.var)   
      result <- cbind(m.mean, mean.se)
      ci <- t(apply(result, 1, FUN=function(x) c(x[1]- qnorm(0.975)*x[2],
                                                 x[1]+ qnorm(0.975)*x[2])))
      result <- cbind(result, ci)
      colnames(result) <- c("est", "se", "lb", "ub")
    } else{
      result <- matrix(m.mean)
      colnames(result) <- c("est")
    }
    
    
    return(result)
  } 
}

# trimmed mean
trimmed.mean <- function(x1, x2, l, u, n=1e5){
  res <- sapply(1:n, function(i){
    e <- rnorm(1, 0, 1)
    y <- exp(x1 - 0.5 * x2 + e)
    if(log(y) > l & log(y) < u){
      return(y)
    }else{
      return(NA)
    }
  })
  return(mean(res, na.rm = T))
}


# function to summarize all the results
# input: sample size
func_res <- function(n){
  res <- parSapply(cl, 1:reps, function(i){
    # seed
    set.seed(i)
    
    # data for CPM
    data <- gen_data(n=n)
    order.y <- data$y[order(data$y)]
    
    # data for Zeng's model
    data.z <- data
    data.z$y <- ifelse(data.z$y < L, L, data.z$y)
    data.z$y <- ifelse(data.z$y > U, U, data.z$y)
    order.y.z <- data.z$y[order(data.z$y)]
    
    # percentage of censoring
    percent_censor <- 1 - mean(data.z$y > L & data.z$y < U)
    
    # model for CPM
    mod <- orm(y ~ x1 + x2,
               family = probit,
               data = data)
    
    ## point estimation
    beta.est <- mod$coefficients[c("x1", "x2")]
    
    alpha.est <- -sapply(log.y, function(x){
      if(x <= min(data$y)){
        result <- Inf
      }else if(x > max(data$y)){
        result <- -Inf
      }else{
        mod$coef[which(order.y >= x)[1] - 1]
      }
    })
    
    alpha.all.est <- -mod$coefficients[1:(length(order.y)-1)]
    
    ## variance
    var <- vcov(mod, intercepts = 'all')
    beta.var <- diag(var[c("x1", "x2"),c("x1", "x2")])
    alpha.var <- sapply(log.y, function(x){
      if(x <= min(data$y) | x > max(data$y)){
        return(Inf)
      }else{
        return(var[which(order.y >= x)[1] - 1, which(order.y >= x)[1] - 1])
      }
    })
    
    ## MSE (est- true)^2
    beta.mse <- (beta.est - beta.true)^2
    alpha.mse <- (alpha.est - alpha.true)^2
    
    ## expectation
    new.data <- data.frame(x1 = c(0, 1), 
                           x2 = c(0, 0))
    
    mean.res <- mean.orm(mod, new.data, se=T)
    mean <- mean.res[,"est"]
    mean.se <- mean.res[,"se"]
    
    
    # model for Zeng's likelihood
    mod.z <- orm(y ~ x1 + x2,
                 family = probit,
                 data = data.z)
    
    beta.est.z <- mod.z$coefficients[c("x1", "x2")]
    alpha.est.z <- -sapply(log.y, function(x){
      if(x <= min(data.z$y)){
        result <- Inf
      }else if(x > max(data.z$y)){
        result <- -Inf
      }else{
        mod.z$coef[which(unique(order.y.z) >= x)[1] - 1]
      }
    })
    
    alpha.all.est.z <- -mod.z$coefficients[1:(length(unique(order.y.z))-1)]
    
    ## variance
    var.z <- vcov(mod.z, intercepts = 'all')
    beta.var.z <- diag(var.z[c("x1", "x2"),c("x1", "x2")])
    alpha.var.z <- sapply(log.y, function(x){
      if(x <= min(data.z$y) | x > max(data.z$y)){
        return(Inf)
      }else{
        return(var.z[which(unique(order.y.z) >= x)[1] - 1, which(unique(order.y.z) >= x)[1] - 1])
      }
    })
    
    ## MSE (est- true)^2
    beta.mse.z <- (beta.est.z - beta.true)^2
    alpha.mse.z <- (alpha.est.z - alpha.true)^2
    
    ## expectation
    mean.res.z <- mean.zeng(mod.z, new.data, se=T, L=L, U=U)
    mean.z <- mean.res.z[,"est"]
    mean.se.z <- mean.res.z[,"se"]
    
    return(list(beta.est = beta.est,
                alpha.est = alpha.est,
                alpha.all.est = alpha.all.est,
                beta.var = beta.var,
                alpha.var = alpha.var,
                beta.mse = beta.mse,
                alpha.mse = alpha.mse,
                mean = mean,
                mean.se = mean.se,
                y = data$y,
                beta.est.z = beta.est.z,
                alpha.est.z = alpha.est.z,
                alpha.all.est.z = alpha.all.est.z,
                beta.var.z = beta.var.z,
                alpha.var.z = alpha.var.z,
                beta.mse.z = beta.mse.z,
                alpha.mse.z = alpha.mse.z,
                mean.z = mean.z,
                mean.se.z = mean.se.z,
                y.z = unique(data.z$y),
                percent_censor = percent_censor))
  })
  
  result <- list(beta.est = list.rbind(res[1,]),
                 alpha.est = list.rbind(res[2,]),
                 alpha.all.est = list.rbind(res[3,]),
                 beta.var = list.rbind(res[4,]),
                 alpha.var = list.rbind(res[5,]),
                 beta.mse = list.rbind(res[6,]),
                 alpha.mse = list.rbind(res[7,]),
                 mean = list.rbind(res[8,]),
                 mean.se = list.rbind(res[9,]),
                 y = list.rbind(res[10,]),
                 beta.est.z = list.rbind(res[11,]),
                 alpha.est.z = list.rbind(res[12,]),
                 alpha.all.est.z = res[13,],
                 beta.var.z = list.rbind(res[14,]),
                 alpha.var.z = list.rbind(res[15,]),
                 beta.mse.z = list.rbind(res[16,]),
                 alpha.mse.z = list.rbind(res[17,]),
                 mean.z = list.rbind(res[18,]),
                 mean.se.z = list.rbind(res[19,]),
                 y.z = res[20,],
                 percent_censor = list.rbind(res[21,]))
  
  # average of a(y)
  result$diff.alpha <- lapply(1:reps, function(i){
    return(cbind(sort(result$y[i,])[-c(1,ncol(result$y))], 
                 diff(result$alpha.all.est[i,])))
  }) %>% list.rbind %>% 
    as.data.frame() %>% 
    arrange(V1) %>% 
    mutate(V2 = cumsum(V2)/reps + mean(result$alpha.all.est[,1]))
  
  # average of a(y).z
  result$diff.alpha.z <- lapply(1:reps, function(i){
    return(cbind(sort(result$y.z[[i]])[-c(1,length(result$y.z[[i]]))],
                 diff(result$alpha.all.est.z[[i]])))
  }) %>% list.rbind %>%
    as.data.frame() %>%
    arrange(V1) %>%
    mutate(V2 = cumsum(V2)/reps +
             mean(sapply(result$alpha.all.est.z, function(x) x[1])))
  
  return(result)
}

# number of replications
reps <- 1000

# setting
log.y <- c(-2, -0.33, 0.5, 1.33, 3)

new.data <- data.frame(x1 = c(0, 1), 
                       x2 = c(0, 0))

beta.true <- c(1, -0.5)
alpha.true <- log.y
mean.true <- c(exp(1)**0.5, exp(1)**1.5)
mean.true.z <- c(trimmed.mean(x1=0, x2=0, l=L, u=U),
                 trimmed.mean(x1=1, x2=0, l=L, u=U))

##### L = -100, U = 100 #####
L <- -100
U <- 100
result_100 <- func_res(n=100)
result_1000 <- func_res(n=1000)
result_5000 <- func_res(n=5000)

##### L = -10, U = 10 #####
L <- -10
U <- 10
result_100 <- func_res(n=100)
result_1000 <- func_res(n=1000)
result_5000 <- func_res(n=5000)

##### L = -4, U = 4 #####
L <- -4
U <- 4
result_100 <- func_res(n=100)
result_1000 <- func_res(n=1000)
result_5000 <- func_res(n=5000)

##### L = -2, U = 2 #####
L <- -2
U <- 2
result_100 <- func_res(n=100)
result_1000 <- func_res(n=1000)
result_5000 <- func_res(n=5000)

##### L = -0.5, U = 0.5 #####
L <- -0.5
U <- 0.5
result_100 <- func_res(n=100)
result_1000 <- func_res(n=1000)
result_5000 <- func_res(n=5000)
